# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import httpx
import pytest
from respx import MockRouter

from telnyx import Telnyx, AsyncTelnyx
from tests.utils import assert_matches_type
from telnyx._response import (
    BinaryAPIResponse,
    AsyncBinaryAPIResponse,
    StreamedBinaryAPIResponse,
    AsyncStreamedBinaryAPIResponse,
)
from telnyx.types.porting import (
    LoaConfigurationListResponse,
    LoaConfigurationCreateResponse,
    LoaConfigurationUpdateResponse,
    LoaConfigurationRetrieveResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestLoaConfigurations:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_create(self, client: Telnyx) -> None:
        loa_configuration = client.porting.loa_configurations.create(
            address={
                "city": "Austin",
                "country_code": "US",
                "state": "TX",
                "street_address": "600 Congress Avenue",
                "zip_code": "78701",
            },
            company_name="Telnyx",
            contact={
                "email": "testing@telnyx.com",
                "phone_number": "+12003270001",
            },
            logo={"document_id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e"},
            name="My LOA Configuration",
        )
        assert_matches_type(LoaConfigurationCreateResponse, loa_configuration, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_create_with_all_params(self, client: Telnyx) -> None:
        loa_configuration = client.porting.loa_configurations.create(
            address={
                "city": "Austin",
                "country_code": "US",
                "state": "TX",
                "street_address": "600 Congress Avenue",
                "zip_code": "78701",
                "extended_address": "14th Floor",
            },
            company_name="Telnyx",
            contact={
                "email": "testing@telnyx.com",
                "phone_number": "+12003270001",
            },
            logo={"document_id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e"},
            name="My LOA Configuration",
        )
        assert_matches_type(LoaConfigurationCreateResponse, loa_configuration, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_create(self, client: Telnyx) -> None:
        response = client.porting.loa_configurations.with_raw_response.create(
            address={
                "city": "Austin",
                "country_code": "US",
                "state": "TX",
                "street_address": "600 Congress Avenue",
                "zip_code": "78701",
            },
            company_name="Telnyx",
            contact={
                "email": "testing@telnyx.com",
                "phone_number": "+12003270001",
            },
            logo={"document_id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e"},
            name="My LOA Configuration",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        loa_configuration = response.parse()
        assert_matches_type(LoaConfigurationCreateResponse, loa_configuration, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_create(self, client: Telnyx) -> None:
        with client.porting.loa_configurations.with_streaming_response.create(
            address={
                "city": "Austin",
                "country_code": "US",
                "state": "TX",
                "street_address": "600 Congress Avenue",
                "zip_code": "78701",
            },
            company_name="Telnyx",
            contact={
                "email": "testing@telnyx.com",
                "phone_number": "+12003270001",
            },
            logo={"document_id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e"},
            name="My LOA Configuration",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            loa_configuration = response.parse()
            assert_matches_type(LoaConfigurationCreateResponse, loa_configuration, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_retrieve(self, client: Telnyx) -> None:
        loa_configuration = client.porting.loa_configurations.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(LoaConfigurationRetrieveResponse, loa_configuration, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_retrieve(self, client: Telnyx) -> None:
        response = client.porting.loa_configurations.with_raw_response.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        loa_configuration = response.parse()
        assert_matches_type(LoaConfigurationRetrieveResponse, loa_configuration, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_retrieve(self, client: Telnyx) -> None:
        with client.porting.loa_configurations.with_streaming_response.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            loa_configuration = response.parse()
            assert_matches_type(LoaConfigurationRetrieveResponse, loa_configuration, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_retrieve(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.porting.loa_configurations.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_update(self, client: Telnyx) -> None:
        loa_configuration = client.porting.loa_configurations.update(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            address={
                "city": "Austin",
                "country_code": "US",
                "state": "TX",
                "street_address": "600 Congress Avenue",
                "zip_code": "78701",
            },
            company_name="Telnyx",
            contact={
                "email": "testing@telnyx.com",
                "phone_number": "+12003270001",
            },
            logo={"document_id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e"},
            name="My LOA Configuration",
        )
        assert_matches_type(LoaConfigurationUpdateResponse, loa_configuration, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_update_with_all_params(self, client: Telnyx) -> None:
        loa_configuration = client.porting.loa_configurations.update(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            address={
                "city": "Austin",
                "country_code": "US",
                "state": "TX",
                "street_address": "600 Congress Avenue",
                "zip_code": "78701",
                "extended_address": "14th Floor",
            },
            company_name="Telnyx",
            contact={
                "email": "testing@telnyx.com",
                "phone_number": "+12003270001",
            },
            logo={"document_id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e"},
            name="My LOA Configuration",
        )
        assert_matches_type(LoaConfigurationUpdateResponse, loa_configuration, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_update(self, client: Telnyx) -> None:
        response = client.porting.loa_configurations.with_raw_response.update(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            address={
                "city": "Austin",
                "country_code": "US",
                "state": "TX",
                "street_address": "600 Congress Avenue",
                "zip_code": "78701",
            },
            company_name="Telnyx",
            contact={
                "email": "testing@telnyx.com",
                "phone_number": "+12003270001",
            },
            logo={"document_id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e"},
            name="My LOA Configuration",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        loa_configuration = response.parse()
        assert_matches_type(LoaConfigurationUpdateResponse, loa_configuration, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_update(self, client: Telnyx) -> None:
        with client.porting.loa_configurations.with_streaming_response.update(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            address={
                "city": "Austin",
                "country_code": "US",
                "state": "TX",
                "street_address": "600 Congress Avenue",
                "zip_code": "78701",
            },
            company_name="Telnyx",
            contact={
                "email": "testing@telnyx.com",
                "phone_number": "+12003270001",
            },
            logo={"document_id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e"},
            name="My LOA Configuration",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            loa_configuration = response.parse()
            assert_matches_type(LoaConfigurationUpdateResponse, loa_configuration, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_update(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.porting.loa_configurations.with_raw_response.update(
                id="",
                address={
                    "city": "Austin",
                    "country_code": "US",
                    "state": "TX",
                    "street_address": "600 Congress Avenue",
                    "zip_code": "78701",
                },
                company_name="Telnyx",
                contact={
                    "email": "testing@telnyx.com",
                    "phone_number": "+12003270001",
                },
                logo={"document_id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e"},
                name="My LOA Configuration",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list(self, client: Telnyx) -> None:
        loa_configuration = client.porting.loa_configurations.list()
        assert_matches_type(LoaConfigurationListResponse, loa_configuration, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list_with_all_params(self, client: Telnyx) -> None:
        loa_configuration = client.porting.loa_configurations.list(
            page={
                "number": 1,
                "size": 1,
            },
        )
        assert_matches_type(LoaConfigurationListResponse, loa_configuration, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_list(self, client: Telnyx) -> None:
        response = client.porting.loa_configurations.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        loa_configuration = response.parse()
        assert_matches_type(LoaConfigurationListResponse, loa_configuration, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_list(self, client: Telnyx) -> None:
        with client.porting.loa_configurations.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            loa_configuration = response.parse()
            assert_matches_type(LoaConfigurationListResponse, loa_configuration, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_delete(self, client: Telnyx) -> None:
        loa_configuration = client.porting.loa_configurations.delete(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert loa_configuration is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_delete(self, client: Telnyx) -> None:
        response = client.porting.loa_configurations.with_raw_response.delete(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        loa_configuration = response.parse()
        assert loa_configuration is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_delete(self, client: Telnyx) -> None:
        with client.porting.loa_configurations.with_streaming_response.delete(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            loa_configuration = response.parse()
            assert loa_configuration is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_delete(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.porting.loa_configurations.with_raw_response.delete(
                "",
            )

    @parametrize
    @pytest.mark.respx(base_url=base_url)
    def test_method_preview_0(self, client: Telnyx, respx_mock: MockRouter) -> None:
        respx_mock.post("/porting/loa_configuration/preview").mock(
            return_value=httpx.Response(200, json={"foo": "bar"})
        )
        loa_configuration = client.porting.loa_configurations.preview_0(
            address={
                "city": "Austin",
                "country_code": "US",
                "state": "TX",
                "street_address": "600 Congress Avenue",
                "zip_code": "78701",
            },
            company_name="Telnyx",
            contact={
                "email": "testing@telnyx.com",
                "phone_number": "+12003270001",
            },
            logo={"document_id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e"},
            name="My LOA Configuration",
        )
        assert loa_configuration.is_closed
        assert loa_configuration.json() == {"foo": "bar"}
        assert cast(Any, loa_configuration.is_closed) is True
        assert isinstance(loa_configuration, BinaryAPIResponse)

    @parametrize
    @pytest.mark.respx(base_url=base_url)
    def test_method_preview_0_with_all_params(self, client: Telnyx, respx_mock: MockRouter) -> None:
        respx_mock.post("/porting/loa_configuration/preview").mock(
            return_value=httpx.Response(200, json={"foo": "bar"})
        )
        loa_configuration = client.porting.loa_configurations.preview_0(
            address={
                "city": "Austin",
                "country_code": "US",
                "state": "TX",
                "street_address": "600 Congress Avenue",
                "zip_code": "78701",
                "extended_address": "14th Floor",
            },
            company_name="Telnyx",
            contact={
                "email": "testing@telnyx.com",
                "phone_number": "+12003270001",
            },
            logo={"document_id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e"},
            name="My LOA Configuration",
        )
        assert loa_configuration.is_closed
        assert loa_configuration.json() == {"foo": "bar"}
        assert cast(Any, loa_configuration.is_closed) is True
        assert isinstance(loa_configuration, BinaryAPIResponse)

    @parametrize
    @pytest.mark.respx(base_url=base_url)
    def test_raw_response_preview_0(self, client: Telnyx, respx_mock: MockRouter) -> None:
        respx_mock.post("/porting/loa_configuration/preview").mock(
            return_value=httpx.Response(200, json={"foo": "bar"})
        )

        loa_configuration = client.porting.loa_configurations.with_raw_response.preview_0(
            address={
                "city": "Austin",
                "country_code": "US",
                "state": "TX",
                "street_address": "600 Congress Avenue",
                "zip_code": "78701",
            },
            company_name="Telnyx",
            contact={
                "email": "testing@telnyx.com",
                "phone_number": "+12003270001",
            },
            logo={"document_id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e"},
            name="My LOA Configuration",
        )

        assert loa_configuration.is_closed is True
        assert loa_configuration.http_request.headers.get("X-Stainless-Lang") == "python"
        assert loa_configuration.json() == {"foo": "bar"}
        assert isinstance(loa_configuration, BinaryAPIResponse)

    @parametrize
    @pytest.mark.respx(base_url=base_url)
    def test_streaming_response_preview_0(self, client: Telnyx, respx_mock: MockRouter) -> None:
        respx_mock.post("/porting/loa_configuration/preview").mock(
            return_value=httpx.Response(200, json={"foo": "bar"})
        )
        with client.porting.loa_configurations.with_streaming_response.preview_0(
            address={
                "city": "Austin",
                "country_code": "US",
                "state": "TX",
                "street_address": "600 Congress Avenue",
                "zip_code": "78701",
            },
            company_name="Telnyx",
            contact={
                "email": "testing@telnyx.com",
                "phone_number": "+12003270001",
            },
            logo={"document_id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e"},
            name="My LOA Configuration",
        ) as loa_configuration:
            assert not loa_configuration.is_closed
            assert loa_configuration.http_request.headers.get("X-Stainless-Lang") == "python"

            assert loa_configuration.json() == {"foo": "bar"}
            assert cast(Any, loa_configuration.is_closed) is True
            assert isinstance(loa_configuration, StreamedBinaryAPIResponse)

        assert cast(Any, loa_configuration.is_closed) is True

    @parametrize
    @pytest.mark.respx(base_url=base_url)
    def test_method_preview_1(self, client: Telnyx, respx_mock: MockRouter) -> None:
        respx_mock.get("/porting/loa_configurations/182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e/preview").mock(
            return_value=httpx.Response(200, json={"foo": "bar"})
        )
        loa_configuration = client.porting.loa_configurations.preview_1(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert loa_configuration.is_closed
        assert loa_configuration.json() == {"foo": "bar"}
        assert cast(Any, loa_configuration.is_closed) is True
        assert isinstance(loa_configuration, BinaryAPIResponse)

    @parametrize
    @pytest.mark.respx(base_url=base_url)
    def test_raw_response_preview_1(self, client: Telnyx, respx_mock: MockRouter) -> None:
        respx_mock.get("/porting/loa_configurations/182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e/preview").mock(
            return_value=httpx.Response(200, json={"foo": "bar"})
        )

        loa_configuration = client.porting.loa_configurations.with_raw_response.preview_1(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert loa_configuration.is_closed is True
        assert loa_configuration.http_request.headers.get("X-Stainless-Lang") == "python"
        assert loa_configuration.json() == {"foo": "bar"}
        assert isinstance(loa_configuration, BinaryAPIResponse)

    @parametrize
    @pytest.mark.respx(base_url=base_url)
    def test_streaming_response_preview_1(self, client: Telnyx, respx_mock: MockRouter) -> None:
        respx_mock.get("/porting/loa_configurations/182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e/preview").mock(
            return_value=httpx.Response(200, json={"foo": "bar"})
        )
        with client.porting.loa_configurations.with_streaming_response.preview_1(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as loa_configuration:
            assert not loa_configuration.is_closed
            assert loa_configuration.http_request.headers.get("X-Stainless-Lang") == "python"

            assert loa_configuration.json() == {"foo": "bar"}
            assert cast(Any, loa_configuration.is_closed) is True
            assert isinstance(loa_configuration, StreamedBinaryAPIResponse)

        assert cast(Any, loa_configuration.is_closed) is True

    @parametrize
    @pytest.mark.respx(base_url=base_url)
    def test_path_params_preview_1(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.porting.loa_configurations.with_raw_response.preview_1(
                "",
            )


class TestAsyncLoaConfigurations:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_create(self, async_client: AsyncTelnyx) -> None:
        loa_configuration = await async_client.porting.loa_configurations.create(
            address={
                "city": "Austin",
                "country_code": "US",
                "state": "TX",
                "street_address": "600 Congress Avenue",
                "zip_code": "78701",
            },
            company_name="Telnyx",
            contact={
                "email": "testing@telnyx.com",
                "phone_number": "+12003270001",
            },
            logo={"document_id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e"},
            name="My LOA Configuration",
        )
        assert_matches_type(LoaConfigurationCreateResponse, loa_configuration, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncTelnyx) -> None:
        loa_configuration = await async_client.porting.loa_configurations.create(
            address={
                "city": "Austin",
                "country_code": "US",
                "state": "TX",
                "street_address": "600 Congress Avenue",
                "zip_code": "78701",
                "extended_address": "14th Floor",
            },
            company_name="Telnyx",
            contact={
                "email": "testing@telnyx.com",
                "phone_number": "+12003270001",
            },
            logo={"document_id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e"},
            name="My LOA Configuration",
        )
        assert_matches_type(LoaConfigurationCreateResponse, loa_configuration, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_create(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.porting.loa_configurations.with_raw_response.create(
            address={
                "city": "Austin",
                "country_code": "US",
                "state": "TX",
                "street_address": "600 Congress Avenue",
                "zip_code": "78701",
            },
            company_name="Telnyx",
            contact={
                "email": "testing@telnyx.com",
                "phone_number": "+12003270001",
            },
            logo={"document_id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e"},
            name="My LOA Configuration",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        loa_configuration = await response.parse()
        assert_matches_type(LoaConfigurationCreateResponse, loa_configuration, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncTelnyx) -> None:
        async with async_client.porting.loa_configurations.with_streaming_response.create(
            address={
                "city": "Austin",
                "country_code": "US",
                "state": "TX",
                "street_address": "600 Congress Avenue",
                "zip_code": "78701",
            },
            company_name="Telnyx",
            contact={
                "email": "testing@telnyx.com",
                "phone_number": "+12003270001",
            },
            logo={"document_id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e"},
            name="My LOA Configuration",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            loa_configuration = await response.parse()
            assert_matches_type(LoaConfigurationCreateResponse, loa_configuration, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_retrieve(self, async_client: AsyncTelnyx) -> None:
        loa_configuration = await async_client.porting.loa_configurations.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(LoaConfigurationRetrieveResponse, loa_configuration, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.porting.loa_configurations.with_raw_response.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        loa_configuration = await response.parse()
        assert_matches_type(LoaConfigurationRetrieveResponse, loa_configuration, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncTelnyx) -> None:
        async with async_client.porting.loa_configurations.with_streaming_response.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            loa_configuration = await response.parse()
            assert_matches_type(LoaConfigurationRetrieveResponse, loa_configuration, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.porting.loa_configurations.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_update(self, async_client: AsyncTelnyx) -> None:
        loa_configuration = await async_client.porting.loa_configurations.update(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            address={
                "city": "Austin",
                "country_code": "US",
                "state": "TX",
                "street_address": "600 Congress Avenue",
                "zip_code": "78701",
            },
            company_name="Telnyx",
            contact={
                "email": "testing@telnyx.com",
                "phone_number": "+12003270001",
            },
            logo={"document_id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e"},
            name="My LOA Configuration",
        )
        assert_matches_type(LoaConfigurationUpdateResponse, loa_configuration, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_update_with_all_params(self, async_client: AsyncTelnyx) -> None:
        loa_configuration = await async_client.porting.loa_configurations.update(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            address={
                "city": "Austin",
                "country_code": "US",
                "state": "TX",
                "street_address": "600 Congress Avenue",
                "zip_code": "78701",
                "extended_address": "14th Floor",
            },
            company_name="Telnyx",
            contact={
                "email": "testing@telnyx.com",
                "phone_number": "+12003270001",
            },
            logo={"document_id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e"},
            name="My LOA Configuration",
        )
        assert_matches_type(LoaConfigurationUpdateResponse, loa_configuration, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_update(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.porting.loa_configurations.with_raw_response.update(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            address={
                "city": "Austin",
                "country_code": "US",
                "state": "TX",
                "street_address": "600 Congress Avenue",
                "zip_code": "78701",
            },
            company_name="Telnyx",
            contact={
                "email": "testing@telnyx.com",
                "phone_number": "+12003270001",
            },
            logo={"document_id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e"},
            name="My LOA Configuration",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        loa_configuration = await response.parse()
        assert_matches_type(LoaConfigurationUpdateResponse, loa_configuration, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_update(self, async_client: AsyncTelnyx) -> None:
        async with async_client.porting.loa_configurations.with_streaming_response.update(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            address={
                "city": "Austin",
                "country_code": "US",
                "state": "TX",
                "street_address": "600 Congress Avenue",
                "zip_code": "78701",
            },
            company_name="Telnyx",
            contact={
                "email": "testing@telnyx.com",
                "phone_number": "+12003270001",
            },
            logo={"document_id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e"},
            name="My LOA Configuration",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            loa_configuration = await response.parse()
            assert_matches_type(LoaConfigurationUpdateResponse, loa_configuration, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_update(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.porting.loa_configurations.with_raw_response.update(
                id="",
                address={
                    "city": "Austin",
                    "country_code": "US",
                    "state": "TX",
                    "street_address": "600 Congress Avenue",
                    "zip_code": "78701",
                },
                company_name="Telnyx",
                contact={
                    "email": "testing@telnyx.com",
                    "phone_number": "+12003270001",
                },
                logo={"document_id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e"},
                name="My LOA Configuration",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list(self, async_client: AsyncTelnyx) -> None:
        loa_configuration = await async_client.porting.loa_configurations.list()
        assert_matches_type(LoaConfigurationListResponse, loa_configuration, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncTelnyx) -> None:
        loa_configuration = await async_client.porting.loa_configurations.list(
            page={
                "number": 1,
                "size": 1,
            },
        )
        assert_matches_type(LoaConfigurationListResponse, loa_configuration, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.porting.loa_configurations.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        loa_configuration = await response.parse()
        assert_matches_type(LoaConfigurationListResponse, loa_configuration, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncTelnyx) -> None:
        async with async_client.porting.loa_configurations.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            loa_configuration = await response.parse()
            assert_matches_type(LoaConfigurationListResponse, loa_configuration, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_delete(self, async_client: AsyncTelnyx) -> None:
        loa_configuration = await async_client.porting.loa_configurations.delete(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert loa_configuration is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.porting.loa_configurations.with_raw_response.delete(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        loa_configuration = await response.parse()
        assert loa_configuration is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncTelnyx) -> None:
        async with async_client.porting.loa_configurations.with_streaming_response.delete(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            loa_configuration = await response.parse()
            assert loa_configuration is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_delete(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.porting.loa_configurations.with_raw_response.delete(
                "",
            )

    @parametrize
    @pytest.mark.respx(base_url=base_url)
    async def test_method_preview_0(self, async_client: AsyncTelnyx, respx_mock: MockRouter) -> None:
        respx_mock.post("/porting/loa_configuration/preview").mock(
            return_value=httpx.Response(200, json={"foo": "bar"})
        )
        loa_configuration = await async_client.porting.loa_configurations.preview_0(
            address={
                "city": "Austin",
                "country_code": "US",
                "state": "TX",
                "street_address": "600 Congress Avenue",
                "zip_code": "78701",
            },
            company_name="Telnyx",
            contact={
                "email": "testing@telnyx.com",
                "phone_number": "+12003270001",
            },
            logo={"document_id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e"},
            name="My LOA Configuration",
        )
        assert loa_configuration.is_closed
        assert await loa_configuration.json() == {"foo": "bar"}
        assert cast(Any, loa_configuration.is_closed) is True
        assert isinstance(loa_configuration, AsyncBinaryAPIResponse)

    @parametrize
    @pytest.mark.respx(base_url=base_url)
    async def test_method_preview_0_with_all_params(self, async_client: AsyncTelnyx, respx_mock: MockRouter) -> None:
        respx_mock.post("/porting/loa_configuration/preview").mock(
            return_value=httpx.Response(200, json={"foo": "bar"})
        )
        loa_configuration = await async_client.porting.loa_configurations.preview_0(
            address={
                "city": "Austin",
                "country_code": "US",
                "state": "TX",
                "street_address": "600 Congress Avenue",
                "zip_code": "78701",
                "extended_address": "14th Floor",
            },
            company_name="Telnyx",
            contact={
                "email": "testing@telnyx.com",
                "phone_number": "+12003270001",
            },
            logo={"document_id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e"},
            name="My LOA Configuration",
        )
        assert loa_configuration.is_closed
        assert await loa_configuration.json() == {"foo": "bar"}
        assert cast(Any, loa_configuration.is_closed) is True
        assert isinstance(loa_configuration, AsyncBinaryAPIResponse)

    @parametrize
    @pytest.mark.respx(base_url=base_url)
    async def test_raw_response_preview_0(self, async_client: AsyncTelnyx, respx_mock: MockRouter) -> None:
        respx_mock.post("/porting/loa_configuration/preview").mock(
            return_value=httpx.Response(200, json={"foo": "bar"})
        )

        loa_configuration = await async_client.porting.loa_configurations.with_raw_response.preview_0(
            address={
                "city": "Austin",
                "country_code": "US",
                "state": "TX",
                "street_address": "600 Congress Avenue",
                "zip_code": "78701",
            },
            company_name="Telnyx",
            contact={
                "email": "testing@telnyx.com",
                "phone_number": "+12003270001",
            },
            logo={"document_id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e"},
            name="My LOA Configuration",
        )

        assert loa_configuration.is_closed is True
        assert loa_configuration.http_request.headers.get("X-Stainless-Lang") == "python"
        assert await loa_configuration.json() == {"foo": "bar"}
        assert isinstance(loa_configuration, AsyncBinaryAPIResponse)

    @parametrize
    @pytest.mark.respx(base_url=base_url)
    async def test_streaming_response_preview_0(self, async_client: AsyncTelnyx, respx_mock: MockRouter) -> None:
        respx_mock.post("/porting/loa_configuration/preview").mock(
            return_value=httpx.Response(200, json={"foo": "bar"})
        )
        async with async_client.porting.loa_configurations.with_streaming_response.preview_0(
            address={
                "city": "Austin",
                "country_code": "US",
                "state": "TX",
                "street_address": "600 Congress Avenue",
                "zip_code": "78701",
            },
            company_name="Telnyx",
            contact={
                "email": "testing@telnyx.com",
                "phone_number": "+12003270001",
            },
            logo={"document_id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e"},
            name="My LOA Configuration",
        ) as loa_configuration:
            assert not loa_configuration.is_closed
            assert loa_configuration.http_request.headers.get("X-Stainless-Lang") == "python"

            assert await loa_configuration.json() == {"foo": "bar"}
            assert cast(Any, loa_configuration.is_closed) is True
            assert isinstance(loa_configuration, AsyncStreamedBinaryAPIResponse)

        assert cast(Any, loa_configuration.is_closed) is True

    @parametrize
    @pytest.mark.respx(base_url=base_url)
    async def test_method_preview_1(self, async_client: AsyncTelnyx, respx_mock: MockRouter) -> None:
        respx_mock.get("/porting/loa_configurations/182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e/preview").mock(
            return_value=httpx.Response(200, json={"foo": "bar"})
        )
        loa_configuration = await async_client.porting.loa_configurations.preview_1(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert loa_configuration.is_closed
        assert await loa_configuration.json() == {"foo": "bar"}
        assert cast(Any, loa_configuration.is_closed) is True
        assert isinstance(loa_configuration, AsyncBinaryAPIResponse)

    @parametrize
    @pytest.mark.respx(base_url=base_url)
    async def test_raw_response_preview_1(self, async_client: AsyncTelnyx, respx_mock: MockRouter) -> None:
        respx_mock.get("/porting/loa_configurations/182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e/preview").mock(
            return_value=httpx.Response(200, json={"foo": "bar"})
        )

        loa_configuration = await async_client.porting.loa_configurations.with_raw_response.preview_1(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert loa_configuration.is_closed is True
        assert loa_configuration.http_request.headers.get("X-Stainless-Lang") == "python"
        assert await loa_configuration.json() == {"foo": "bar"}
        assert isinstance(loa_configuration, AsyncBinaryAPIResponse)

    @parametrize
    @pytest.mark.respx(base_url=base_url)
    async def test_streaming_response_preview_1(self, async_client: AsyncTelnyx, respx_mock: MockRouter) -> None:
        respx_mock.get("/porting/loa_configurations/182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e/preview").mock(
            return_value=httpx.Response(200, json={"foo": "bar"})
        )
        async with async_client.porting.loa_configurations.with_streaming_response.preview_1(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as loa_configuration:
            assert not loa_configuration.is_closed
            assert loa_configuration.http_request.headers.get("X-Stainless-Lang") == "python"

            assert await loa_configuration.json() == {"foo": "bar"}
            assert cast(Any, loa_configuration.is_closed) is True
            assert isinstance(loa_configuration, AsyncStreamedBinaryAPIResponse)

        assert cast(Any, loa_configuration.is_closed) is True

    @parametrize
    @pytest.mark.respx(base_url=base_url)
    async def test_path_params_preview_1(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.porting.loa_configurations.with_raw_response.preview_1(
                "",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from .address import Address
from .._models import BaseModel
from .pagination_meta import PaginationMeta

__all__ = ["AddressListResponse"]


class AddressListResponse(BaseModel):
    data: Optional[List[Address]] = None

    meta: Optional[PaginationMeta] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime

from .._models import BaseModel

__all__ = ["CampaignSubmitAppealResponse"]


class CampaignSubmitAppealResponse(BaseModel):
    appealed_at: Optional[datetime] = None
    """Timestamp when the appeal was submitted"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import TypedDict

from .._types import SequenceNotStr
from .dtmf_type import DtmfType
from .encrypted_media import EncryptedMedia
from .inbound_fqdn_param import InboundFqdnParam
from .transport_protocol import TransportProtocol
from .anchorsite_override import AnchorsiteOverride
from .outbound_fqdn_param import OutboundFqdnParam
from .webhook_api_version import WebhookAPIVersion
from .connection_rtcp_settings_param import ConnectionRtcpSettingsParam

__all__ = ["FqdnConnectionUpdateParams"]


class FqdnConnectionUpdateParams(TypedDict, total=False):
    active: bool
    """Defaults to true"""

    anchorsite_override: AnchorsiteOverride
    """
    `Latency` directs Telnyx to route media through the site with the lowest
    round-trip time to the user's connection. Telnyx calculates this time using ICMP
    ping messages. This can be disabled by specifying a site to handle all media.
    """

    android_push_credential_id: Optional[str]
    """The uuid of the push credential for Android"""

    connection_name: str
    """A user-assigned name to help manage the connection."""

    default_on_hold_comfort_noise_enabled: bool
    """When enabled, Telnyx will generate comfort noise when you place the call on
    hold.

    If disabled, you will need to generate comfort noise or on hold music to avoid
    RTP timeout.
    """

    dtmf_type: DtmfType
    """Sets the type of DTMF digits sent from Telnyx to this Connection.

    Note that DTMF digits sent to Telnyx will be accepted in all formats.
    """

    encode_contact_header_enabled: bool
    """
    Encode the SIP contact header sent by Telnyx to avoid issues for NAT or ALG
    scenarios.
    """

    encrypted_media: Optional[EncryptedMedia]
    """Enable use of SRTP for encryption.

    Cannot be set if the transport_portocol is TLS.
    """

    inbound: InboundFqdnParam

    ios_push_credential_id: Optional[str]
    """The uuid of the push credential for Ios"""

    onnet_t38_passthrough_enabled: bool
    """
    Enable on-net T38 if you prefer that the sender and receiver negotiate T38
    directly when both are on the Telnyx network. If this is disabled, Telnyx will
    be able to use T38 on just one leg of the call according to each leg's settings.
    """

    outbound: OutboundFqdnParam

    rtcp_settings: ConnectionRtcpSettingsParam

    tags: SequenceNotStr[str]
    """Tags associated with the connection."""

    transport_protocol: TransportProtocol
    """One of UDP, TLS, or TCP.

    Applies only to connections with IP authentication or FQDN authentication.
    """

    webhook_api_version: WebhookAPIVersion
    """Determines which webhook format will be used, Telnyx API v1 or v2."""

    webhook_event_failover_url: Optional[str]
    """
    The failover URL where webhooks related to this connection will be sent if
    sending to the primary URL fails. Must include a scheme, such as 'https'.
    """

    webhook_event_url: str
    """The URL where webhooks related to this connection will be sent.

    Must include a scheme, such as 'https'.
    """

    webhook_timeout_secs: Optional[int]
    """Specifies how many seconds to wait before timing out a webhook."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .porting_report import PortingReport as PortingReport
from .event_list_params import EventListParams as EventListParams
from .report_list_params import ReportListParams as ReportListParams
from .event_list_response import EventListResponse as EventListResponse
from .report_create_params import ReportCreateParams as ReportCreateParams
from .report_list_response import ReportListResponse as ReportListResponse
from .report_create_response import ReportCreateResponse as ReportCreateResponse
from .event_retrieve_response import EventRetrieveResponse as EventRetrieveResponse
from .report_retrieve_response import ReportRetrieveResponse as ReportRetrieveResponse
from .porting_loa_configuration import PortingLoaConfiguration as PortingLoaConfiguration
from .loa_configuration_list_params import LoaConfigurationListParams as LoaConfigurationListParams
from .loa_configuration_create_params import LoaConfigurationCreateParams as LoaConfigurationCreateParams
from .loa_configuration_list_response import LoaConfigurationListResponse as LoaConfigurationListResponse
from .loa_configuration_update_params import LoaConfigurationUpdateParams as LoaConfigurationUpdateParams
from .export_porting_orders_csv_report import ExportPortingOrdersCsvReport as ExportPortingOrdersCsvReport
from .loa_configuration_create_response import LoaConfigurationCreateResponse as LoaConfigurationCreateResponse
from .loa_configuration_update_response import LoaConfigurationUpdateResponse as LoaConfigurationUpdateResponse
from .loa_configuration_preview_0_params import LoaConfigurationPreview0Params as LoaConfigurationPreview0Params
from .loa_configuration_retrieve_response import LoaConfigurationRetrieveResponse as LoaConfigurationRetrieveResponse
from .export_porting_orders_csv_report_param import (
    ExportPortingOrdersCsvReportParam as ExportPortingOrdersCsvReportParam,
)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options
from ...types.porting_orders import comment_list_params, comment_create_params
from ...types.porting_orders.comment_list_response import CommentListResponse
from ...types.porting_orders.comment_create_response import CommentCreateResponse

__all__ = ["CommentsResource", "AsyncCommentsResource"]


class CommentsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> CommentsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return CommentsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> CommentsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return CommentsResourceWithStreamingResponse(self)

    def create(
        self,
        id: str,
        *,
        body: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CommentCreateResponse:
        """
        Creates a new comment for a porting order.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._post(
            f"/porting_orders/{id}/comments",
            body=maybe_transform({"body": body}, comment_create_params.CommentCreateParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CommentCreateResponse,
        )

    def list(
        self,
        id: str,
        *,
        page: comment_list_params.Page | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CommentListResponse:
        """
        Returns a list of all comments of a porting order.

        Args:
          page: Consolidated page parameter (deepObject style). Originally: page[size],
              page[number]

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/porting_orders/{id}/comments",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"page": page}, comment_list_params.CommentListParams),
            ),
            cast_to=CommentListResponse,
        )


class AsyncCommentsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncCommentsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncCommentsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncCommentsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncCommentsResourceWithStreamingResponse(self)

    async def create(
        self,
        id: str,
        *,
        body: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CommentCreateResponse:
        """
        Creates a new comment for a porting order.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._post(
            f"/porting_orders/{id}/comments",
            body=await async_maybe_transform({"body": body}, comment_create_params.CommentCreateParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CommentCreateResponse,
        )

    async def list(
        self,
        id: str,
        *,
        page: comment_list_params.Page | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CommentListResponse:
        """
        Returns a list of all comments of a porting order.

        Args:
          page: Consolidated page parameter (deepObject style). Originally: page[size],
              page[number]

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/porting_orders/{id}/comments",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform({"page": page}, comment_list_params.CommentListParams),
            ),
            cast_to=CommentListResponse,
        )


class CommentsResourceWithRawResponse:
    def __init__(self, comments: CommentsResource) -> None:
        self._comments = comments

        self.create = to_raw_response_wrapper(
            comments.create,
        )
        self.list = to_raw_response_wrapper(
            comments.list,
        )


class AsyncCommentsResourceWithRawResponse:
    def __init__(self, comments: AsyncCommentsResource) -> None:
        self._comments = comments

        self.create = async_to_raw_response_wrapper(
            comments.create,
        )
        self.list = async_to_raw_response_wrapper(
            comments.list,
        )


class CommentsResourceWithStreamingResponse:
    def __init__(self, comments: CommentsResource) -> None:
        self._comments = comments

        self.create = to_streamed_response_wrapper(
            comments.create,
        )
        self.list = to_streamed_response_wrapper(
            comments.list,
        )


class AsyncCommentsResourceWithStreamingResponse:
    def __init__(self, comments: AsyncCommentsResource) -> None:
        self._comments = comments

        self.create = async_to_streamed_response_wrapper(
            comments.create,
        )
        self.list = async_to_streamed_response_wrapper(
            comments.list,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel

__all__ = ["NumberPoolSettings"]


class NumberPoolSettings(BaseModel):
    long_code_weight: float
    """
    Defines the probability weight for a Long Code number to be selected when
    sending a message. The higher the weight the higher the probability. The sum of
    the weights for all number types does not necessarily need to add to 100. Weight
    must be a non-negative number, and when equal to zero it will remove the number
    type from the pool.
    """

    skip_unhealthy: bool
    """
    If set to true all unhealthy numbers will be automatically excluded from the
    pool. Health metrics per number are calculated on a regular basis, taking into
    account the deliverability rate and the amount of messages marked as spam by
    upstream carriers. Numbers with a deliverability rate below 25% or spam ratio
    over 75% will be considered unhealthy.
    """

    toll_free_weight: float
    """
    Defines the probability weight for a Toll Free number to be selected when
    sending a message. The higher the weight the higher the probability. The sum of
    the weights for all number types does not necessarily need to add to 100. Weight
    must be a non-negative number, and when equal to zero it will remove the number
    type from the pool.
    """

    geomatch: Optional[bool] = None
    """
    If set to true, Number Pool will try to choose a sending number with the same
    area code as the destination number. If there are no such numbers available, a
    nunber with a different area code will be chosen. Currently only NANP numbers
    are supported.
    """

    sticky_sender: Optional[bool] = None
    """
    If set to true, Number Pool will try to choose the same sending number for all
    messages to a particular recipient. If the sending number becomes unhealthy and
    `skip_unhealthy` is set to true, a new number will be chosen.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime

from ...._models import BaseModel
from .telnyx_conversation_channel import TelnyxConversationChannel

__all__ = ["AssistantTest", "Rubric"]


class Rubric(BaseModel):
    criteria: str
    """
    Specific guidance on how to assess the assistant’s performance for this rubric
    item.
    """

    name: str
    """Label for the evaluation criterion, e.g., Empathy, Accuracy, Clarity."""


class AssistantTest(BaseModel):
    created_at: datetime
    """Timestamp when the test was created."""

    name: str
    """Human-readable name of the test."""

    rubric: List[Rubric]
    """Evaluation criteria used to assess test performance."""

    telnyx_conversation_channel: TelnyxConversationChannel
    """Communication channel used for test execution."""

    test_id: str
    """Unique identifier for the assistant test."""

    description: Optional[str] = None
    """Detailed description of the test's purpose and scope."""

    destination: Optional[str] = None
    """Target destination for test conversations."""

    instructions: Optional[str] = None
    """Detailed test scenario instructions and objectives."""

    max_duration_seconds: Optional[int] = None
    """Maximum allowed duration for test execution in seconds."""

    test_suite: Optional[str] = None
    """Test suite grouping for organizational purposes."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from typing_extensions import TypedDict

__all__ = ["InsightUpdateParams"]


class InsightUpdateParams(TypedDict, total=False):
    instructions: str

    json_schema: Union[str, object]

    name: str

    webhook: str



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

__all__ = ["LogMessageListParams", "Filter", "FilterTelephoneNumber", "Page"]


class LogMessageListParams(TypedDict, total=False):
    filter: Filter
    """Filter parameter for log messages (deepObject style).

    Supports filtering by external_connection_id and telephone_number with
    eq/contains operations.
    """

    page: Page
    """Consolidated page parameter (deepObject style).

    Originally: page[size], page[number]
    """


class FilterTelephoneNumber(TypedDict, total=False):
    contains: str
    """The partial phone number to filter log messages for. Requires 3-15 digits."""

    eq: str
    """
    The phone number to filter log messages for or "null" to filter for logs without
    a phone number
    """


class Filter(TypedDict, total=False):
    external_connection_id: str
    """
    The external connection ID to filter by or "null" to filter for logs without an
    external connection ID
    """

    telephone_number: FilterTelephoneNumber
    """Telephone number filter operations for log messages.

    Use 'eq' for exact matches or 'contains' for partial matches.
    """


class Page(TypedDict, total=False):
    number: int
    """The page number to load"""

    size: int
    """The size of the page"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from telnyx import Telnyx, AsyncTelnyx
from tests.utils import assert_matches_type
from telnyx._utils import parse_datetime
from telnyx.types.ai import (
    Conversation,
    ConversationListResponse,
    ConversationUpdateResponse,
    ConversationRetrieveResponse,
    ConversationRetrieveConversationsInsightsResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestConversations:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_create(self, client: Telnyx) -> None:
        conversation = client.ai.conversations.create()
        assert_matches_type(Conversation, conversation, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_create_with_all_params(self, client: Telnyx) -> None:
        conversation = client.ai.conversations.create(
            metadata={"foo": "string"},
            name="name",
        )
        assert_matches_type(Conversation, conversation, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_create(self, client: Telnyx) -> None:
        response = client.ai.conversations.with_raw_response.create()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        conversation = response.parse()
        assert_matches_type(Conversation, conversation, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_create(self, client: Telnyx) -> None:
        with client.ai.conversations.with_streaming_response.create() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            conversation = response.parse()
            assert_matches_type(Conversation, conversation, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_retrieve(self, client: Telnyx) -> None:
        conversation = client.ai.conversations.retrieve(
            "conversation_id",
        )
        assert_matches_type(ConversationRetrieveResponse, conversation, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_retrieve(self, client: Telnyx) -> None:
        response = client.ai.conversations.with_raw_response.retrieve(
            "conversation_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        conversation = response.parse()
        assert_matches_type(ConversationRetrieveResponse, conversation, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_retrieve(self, client: Telnyx) -> None:
        with client.ai.conversations.with_streaming_response.retrieve(
            "conversation_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            conversation = response.parse()
            assert_matches_type(ConversationRetrieveResponse, conversation, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_retrieve(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `conversation_id` but received ''"):
            client.ai.conversations.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_update(self, client: Telnyx) -> None:
        conversation = client.ai.conversations.update(
            conversation_id="conversation_id",
        )
        assert_matches_type(ConversationUpdateResponse, conversation, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_update_with_all_params(self, client: Telnyx) -> None:
        conversation = client.ai.conversations.update(
            conversation_id="conversation_id",
            metadata={"foo": "string"},
        )
        assert_matches_type(ConversationUpdateResponse, conversation, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_update(self, client: Telnyx) -> None:
        response = client.ai.conversations.with_raw_response.update(
            conversation_id="conversation_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        conversation = response.parse()
        assert_matches_type(ConversationUpdateResponse, conversation, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_update(self, client: Telnyx) -> None:
        with client.ai.conversations.with_streaming_response.update(
            conversation_id="conversation_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            conversation = response.parse()
            assert_matches_type(ConversationUpdateResponse, conversation, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_update(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `conversation_id` but received ''"):
            client.ai.conversations.with_raw_response.update(
                conversation_id="",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list(self, client: Telnyx) -> None:
        conversation = client.ai.conversations.list()
        assert_matches_type(ConversationListResponse, conversation, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list_with_all_params(self, client: Telnyx) -> None:
        conversation = client.ai.conversations.list(
            id="id",
            created_at="created_at",
            last_message_at="last_message_at",
            limit=1,
            metadata_assistant_id="metadata->assistant_id",
            metadata_call_control_id="metadata->call_control_id",
            metadata_telnyx_agent_target="metadata->telnyx_agent_target",
            metadata_telnyx_conversation_channel="metadata->telnyx_conversation_channel",
            metadata_telnyx_end_user_target="metadata->telnyx_end_user_target",
            name="name",
            or_="or",
            order="order",
        )
        assert_matches_type(ConversationListResponse, conversation, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_list(self, client: Telnyx) -> None:
        response = client.ai.conversations.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        conversation = response.parse()
        assert_matches_type(ConversationListResponse, conversation, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_list(self, client: Telnyx) -> None:
        with client.ai.conversations.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            conversation = response.parse()
            assert_matches_type(ConversationListResponse, conversation, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_delete(self, client: Telnyx) -> None:
        conversation = client.ai.conversations.delete(
            "conversation_id",
        )
        assert conversation is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_delete(self, client: Telnyx) -> None:
        response = client.ai.conversations.with_raw_response.delete(
            "conversation_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        conversation = response.parse()
        assert conversation is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_delete(self, client: Telnyx) -> None:
        with client.ai.conversations.with_streaming_response.delete(
            "conversation_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            conversation = response.parse()
            assert conversation is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_delete(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `conversation_id` but received ''"):
            client.ai.conversations.with_raw_response.delete(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_add_message(self, client: Telnyx) -> None:
        conversation = client.ai.conversations.add_message(
            conversation_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            role="role",
        )
        assert_matches_type(object, conversation, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_add_message_with_all_params(self, client: Telnyx) -> None:
        conversation = client.ai.conversations.add_message(
            conversation_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            role="role",
            content="content",
            metadata={"foo": "string"},
            name="name",
            sent_at=parse_datetime("2019-12-27T18:11:19.117Z"),
            tool_call_id="tool_call_id",
            tool_calls=[{"foo": "bar"}],
            tool_choice="string",
        )
        assert_matches_type(object, conversation, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_add_message(self, client: Telnyx) -> None:
        response = client.ai.conversations.with_raw_response.add_message(
            conversation_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            role="role",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        conversation = response.parse()
        assert_matches_type(object, conversation, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_add_message(self, client: Telnyx) -> None:
        with client.ai.conversations.with_streaming_response.add_message(
            conversation_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            role="role",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            conversation = response.parse()
            assert_matches_type(object, conversation, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_add_message(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `conversation_id` but received ''"):
            client.ai.conversations.with_raw_response.add_message(
                conversation_id="",
                role="role",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_retrieve_conversations_insights(self, client: Telnyx) -> None:
        conversation = client.ai.conversations.retrieve_conversations_insights(
            "conversation_id",
        )
        assert_matches_type(ConversationRetrieveConversationsInsightsResponse, conversation, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_retrieve_conversations_insights(self, client: Telnyx) -> None:
        response = client.ai.conversations.with_raw_response.retrieve_conversations_insights(
            "conversation_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        conversation = response.parse()
        assert_matches_type(ConversationRetrieveConversationsInsightsResponse, conversation, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_retrieve_conversations_insights(self, client: Telnyx) -> None:
        with client.ai.conversations.with_streaming_response.retrieve_conversations_insights(
            "conversation_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            conversation = response.parse()
            assert_matches_type(ConversationRetrieveConversationsInsightsResponse, conversation, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_retrieve_conversations_insights(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `conversation_id` but received ''"):
            client.ai.conversations.with_raw_response.retrieve_conversations_insights(
                "",
            )


class TestAsyncConversations:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_create(self, async_client: AsyncTelnyx) -> None:
        conversation = await async_client.ai.conversations.create()
        assert_matches_type(Conversation, conversation, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncTelnyx) -> None:
        conversation = await async_client.ai.conversations.create(
            metadata={"foo": "string"},
            name="name",
        )
        assert_matches_type(Conversation, conversation, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_create(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.ai.conversations.with_raw_response.create()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        conversation = await response.parse()
        assert_matches_type(Conversation, conversation, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncTelnyx) -> None:
        async with async_client.ai.conversations.with_streaming_response.create() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            conversation = await response.parse()
            assert_matches_type(Conversation, conversation, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_retrieve(self, async_client: AsyncTelnyx) -> None:
        conversation = await async_client.ai.conversations.retrieve(
            "conversation_id",
        )
        assert_matches_type(ConversationRetrieveResponse, conversation, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.ai.conversations.with_raw_response.retrieve(
            "conversation_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        conversation = await response.parse()
        assert_matches_type(ConversationRetrieveResponse, conversation, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncTelnyx) -> None:
        async with async_client.ai.conversations.with_streaming_response.retrieve(
            "conversation_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            conversation = await response.parse()
            assert_matches_type(ConversationRetrieveResponse, conversation, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `conversation_id` but received ''"):
            await async_client.ai.conversations.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_update(self, async_client: AsyncTelnyx) -> None:
        conversation = await async_client.ai.conversations.update(
            conversation_id="conversation_id",
        )
        assert_matches_type(ConversationUpdateResponse, conversation, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_update_with_all_params(self, async_client: AsyncTelnyx) -> None:
        conversation = await async_client.ai.conversations.update(
            conversation_id="conversation_id",
            metadata={"foo": "string"},
        )
        assert_matches_type(ConversationUpdateResponse, conversation, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_update(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.ai.conversations.with_raw_response.update(
            conversation_id="conversation_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        conversation = await response.parse()
        assert_matches_type(ConversationUpdateResponse, conversation, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_update(self, async_client: AsyncTelnyx) -> None:
        async with async_client.ai.conversations.with_streaming_response.update(
            conversation_id="conversation_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            conversation = await response.parse()
            assert_matches_type(ConversationUpdateResponse, conversation, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_update(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `conversation_id` but received ''"):
            await async_client.ai.conversations.with_raw_response.update(
                conversation_id="",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list(self, async_client: AsyncTelnyx) -> None:
        conversation = await async_client.ai.conversations.list()
        assert_matches_type(ConversationListResponse, conversation, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncTelnyx) -> None:
        conversation = await async_client.ai.conversations.list(
            id="id",
            created_at="created_at",
            last_message_at="last_message_at",
            limit=1,
            metadata_assistant_id="metadata->assistant_id",
            metadata_call_control_id="metadata->call_control_id",
            metadata_telnyx_agent_target="metadata->telnyx_agent_target",
            metadata_telnyx_conversation_channel="metadata->telnyx_conversation_channel",
            metadata_telnyx_end_user_target="metadata->telnyx_end_user_target",
            name="name",
            or_="or",
            order="order",
        )
        assert_matches_type(ConversationListResponse, conversation, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.ai.conversations.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        conversation = await response.parse()
        assert_matches_type(ConversationListResponse, conversation, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncTelnyx) -> None:
        async with async_client.ai.conversations.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            conversation = await response.parse()
            assert_matches_type(ConversationListResponse, conversation, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_delete(self, async_client: AsyncTelnyx) -> None:
        conversation = await async_client.ai.conversations.delete(
            "conversation_id",
        )
        assert conversation is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.ai.conversations.with_raw_response.delete(
            "conversation_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        conversation = await response.parse()
        assert conversation is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncTelnyx) -> None:
        async with async_client.ai.conversations.with_streaming_response.delete(
            "conversation_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            conversation = await response.parse()
            assert conversation is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_delete(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `conversation_id` but received ''"):
            await async_client.ai.conversations.with_raw_response.delete(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_add_message(self, async_client: AsyncTelnyx) -> None:
        conversation = await async_client.ai.conversations.add_message(
            conversation_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            role="role",
        )
        assert_matches_type(object, conversation, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_add_message_with_all_params(self, async_client: AsyncTelnyx) -> None:
        conversation = await async_client.ai.conversations.add_message(
            conversation_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            role="role",
            content="content",
            metadata={"foo": "string"},
            name="name",
            sent_at=parse_datetime("2019-12-27T18:11:19.117Z"),
            tool_call_id="tool_call_id",
            tool_calls=[{"foo": "bar"}],
            tool_choice="string",
        )
        assert_matches_type(object, conversation, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_add_message(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.ai.conversations.with_raw_response.add_message(
            conversation_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            role="role",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        conversation = await response.parse()
        assert_matches_type(object, conversation, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_add_message(self, async_client: AsyncTelnyx) -> None:
        async with async_client.ai.conversations.with_streaming_response.add_message(
            conversation_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            role="role",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            conversation = await response.parse()
            assert_matches_type(object, conversation, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_add_message(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `conversation_id` but received ''"):
            await async_client.ai.conversations.with_raw_response.add_message(
                conversation_id="",
                role="role",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_retrieve_conversations_insights(self, async_client: AsyncTelnyx) -> None:
        conversation = await async_client.ai.conversations.retrieve_conversations_insights(
            "conversation_id",
        )
        assert_matches_type(ConversationRetrieveConversationsInsightsResponse, conversation, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_retrieve_conversations_insights(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.ai.conversations.with_raw_response.retrieve_conversations_insights(
            "conversation_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        conversation = await response.parse()
        assert_matches_type(ConversationRetrieveConversationsInsightsResponse, conversation, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_retrieve_conversations_insights(self, async_client: AsyncTelnyx) -> None:
        async with async_client.ai.conversations.with_streaming_response.retrieve_conversations_insights(
            "conversation_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            conversation = await response.parse()
            assert_matches_type(ConversationRetrieveConversationsInsightsResponse, conversation, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_retrieve_conversations_insights(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `conversation_id` but received ''"):
            await async_client.ai.conversations.with_raw_response.retrieve_conversations_insights(
                "",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Annotated, TypedDict

from ..._utils import PropertyInfo

__all__ = ["CsvDownloadCreateParams", "Filter", "FilterVoiceConnectionName"]


class CsvDownloadCreateParams(TypedDict, total=False):
    csv_format: Literal["V1", "V2"]
    """Which format to use when generating the CSV file.

    The default for backwards compatibility is 'V1'
    """

    filter: Filter
    """Consolidated filter parameter (deepObject style).

    Originally: filter[has_bundle], filter[tag], filter[connection_id],
    filter[phone_number], filter[status], filter[voice.connection_name],
    filter[voice.usage_payment_method], filter[billing_group_id],
    filter[emergency_address_id], filter[customer_reference]
    """


class FilterVoiceConnectionName(TypedDict, total=False):
    contains: str
    """Filter contains connection name. Requires at least three characters."""

    ends_with: str
    """Filter ends with connection name. Requires at least three characters."""

    eq: str
    """Filter by connection name."""

    starts_with: str
    """Filter starts with connection name. Requires at least three characters."""


class Filter(TypedDict, total=False):
    billing_group_id: str
    """Filter by the billing_group_id associated with phone numbers.

    To filter to only phone numbers that have no billing group associated them, set
    the value of this filter to the string 'null'.
    """

    connection_id: str
    """Filter by connection_id."""

    customer_reference: str
    """Filter numbers via the customer_reference set."""

    emergency_address_id: str
    """Filter by the emergency_address_id associated with phone numbers.

    To filter only phone numbers that have no emergency address associated with
    them, set the value of this filter to the string 'null'.
    """

    has_bundle: str
    """Filter by phone number that have bundles."""

    phone_number: str
    """Filter by phone number.

    Requires at least three digits. Non-numerical characters will result in no
    values being returned.
    """

    status: Literal[
        "purchase-pending",
        "purchase-failed",
        "port-pending",
        "active",
        "deleted",
        "port-failed",
        "emergency-only",
        "ported-out",
        "port-out-pending",
    ]
    """Filter by phone number status."""

    tag: str
    """Filter by phone number tags."""

    voice_connection_name: Annotated[FilterVoiceConnectionName, PropertyInfo(alias="voice.connection_name")]
    """Filter by voice connection name pattern matching."""

    voice_usage_payment_method: Annotated[
        Literal["pay-per-minute", "channel"], PropertyInfo(alias="voice.usage_payment_method")
    ]
    """Filter by usage_payment_method."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from ..._models import BaseModel
from .mdr_usage_report import MdrUsageReport
from .pagination_meta_reporting import PaginationMetaReporting

__all__ = ["MdrUsageReportListResponse"]


class MdrUsageReportListResponse(BaseModel):
    data: Optional[List[MdrUsageReport]] = None

    meta: Optional[PaginationMetaReporting] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Union
from typing_extensions import TypeAlias

from ...._models import BaseModel
from .tests.test_suites.meta import Meta
from .scheduled_sms_event_response import ScheduledSMSEventResponse
from .scheduled_phone_call_event_response import ScheduledPhoneCallEventResponse

__all__ = ["ScheduledEventListResponse", "Data"]

Data: TypeAlias = Union[ScheduledPhoneCallEventResponse, ScheduledSMSEventResponse]


class ScheduledEventListResponse(BaseModel):
    data: List[Data]

    meta: Meta



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

__all__ = ["TelnyxVoiceSettingsParam"]


class TelnyxVoiceSettingsParam(TypedDict, total=False):
    voice_speed: float
    """The voice speed to be used for the voice.

    The voice speed must be between 0.1 and 2.0. Default value is 1.0.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["SettingsParam"]


class SettingsParam(TypedDict, total=False):
    idp_cert_fingerprint: Required[str]
    """The certificate fingerprint for the identity provider (IdP)"""

    idp_entity_id: Required[str]
    """The Entity ID for the identity provider (IdP)."""

    idp_sso_target_url: Required[str]
    """The SSO target url for the identity provider (IdP)."""

    idp_cert_fingerprint_algorithm: Literal["sha1", "sha256", "sha384", "sha512"]
    """
    The algorithm used to generate the identity provider's (IdP) certificate
    fingerprint
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options
from ...types.porting_orders import action_requirement_list_params, action_requirement_initiate_params
from ...types.porting_orders.action_requirement_list_response import ActionRequirementListResponse
from ...types.porting_orders.action_requirement_initiate_response import ActionRequirementInitiateResponse

__all__ = ["ActionRequirementsResource", "AsyncActionRequirementsResource"]


class ActionRequirementsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> ActionRequirementsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return ActionRequirementsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ActionRequirementsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return ActionRequirementsResourceWithStreamingResponse(self)

    def list(
        self,
        porting_order_id: str,
        *,
        filter: action_requirement_list_params.Filter | Omit = omit,
        page: action_requirement_list_params.Page | Omit = omit,
        sort: action_requirement_list_params.Sort | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ActionRequirementListResponse:
        """
        Returns a list of action requirements for a specific porting order.

        Args:
          filter: Consolidated filter parameter (deepObject style). Originally: filter[id][in][],
              filter[requirement_type_id], filter[action_type], filter[status]

          page: Consolidated page parameter (deepObject style). Originally: page[size],
              page[number]

          sort: Consolidated sort parameter (deepObject style). Originally: sort[value]

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not porting_order_id:
            raise ValueError(f"Expected a non-empty value for `porting_order_id` but received {porting_order_id!r}")
        return self._get(
            f"/porting_orders/{porting_order_id}/action_requirements",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "filter": filter,
                        "page": page,
                        "sort": sort,
                    },
                    action_requirement_list_params.ActionRequirementListParams,
                ),
            ),
            cast_to=ActionRequirementListResponse,
        )

    def initiate(
        self,
        id: str,
        *,
        porting_order_id: str,
        params: action_requirement_initiate_params.Params,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ActionRequirementInitiateResponse:
        """
        Initiates a specific action requirement for a porting order.

        Args:
          params: Required information for initiating the action requirement for AU ID
              verification.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not porting_order_id:
            raise ValueError(f"Expected a non-empty value for `porting_order_id` but received {porting_order_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._post(
            f"/porting_orders/{porting_order_id}/action_requirements/{id}/initiate",
            body=maybe_transform(
                {"params": params}, action_requirement_initiate_params.ActionRequirementInitiateParams
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ActionRequirementInitiateResponse,
        )


class AsyncActionRequirementsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncActionRequirementsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncActionRequirementsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncActionRequirementsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncActionRequirementsResourceWithStreamingResponse(self)

    async def list(
        self,
        porting_order_id: str,
        *,
        filter: action_requirement_list_params.Filter | Omit = omit,
        page: action_requirement_list_params.Page | Omit = omit,
        sort: action_requirement_list_params.Sort | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ActionRequirementListResponse:
        """
        Returns a list of action requirements for a specific porting order.

        Args:
          filter: Consolidated filter parameter (deepObject style). Originally: filter[id][in][],
              filter[requirement_type_id], filter[action_type], filter[status]

          page: Consolidated page parameter (deepObject style). Originally: page[size],
              page[number]

          sort: Consolidated sort parameter (deepObject style). Originally: sort[value]

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not porting_order_id:
            raise ValueError(f"Expected a non-empty value for `porting_order_id` but received {porting_order_id!r}")
        return await self._get(
            f"/porting_orders/{porting_order_id}/action_requirements",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "filter": filter,
                        "page": page,
                        "sort": sort,
                    },
                    action_requirement_list_params.ActionRequirementListParams,
                ),
            ),
            cast_to=ActionRequirementListResponse,
        )

    async def initiate(
        self,
        id: str,
        *,
        porting_order_id: str,
        params: action_requirement_initiate_params.Params,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ActionRequirementInitiateResponse:
        """
        Initiates a specific action requirement for a porting order.

        Args:
          params: Required information for initiating the action requirement for AU ID
              verification.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not porting_order_id:
            raise ValueError(f"Expected a non-empty value for `porting_order_id` but received {porting_order_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._post(
            f"/porting_orders/{porting_order_id}/action_requirements/{id}/initiate",
            body=await async_maybe_transform(
                {"params": params}, action_requirement_initiate_params.ActionRequirementInitiateParams
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ActionRequirementInitiateResponse,
        )


class ActionRequirementsResourceWithRawResponse:
    def __init__(self, action_requirements: ActionRequirementsResource) -> None:
        self._action_requirements = action_requirements

        self.list = to_raw_response_wrapper(
            action_requirements.list,
        )
        self.initiate = to_raw_response_wrapper(
            action_requirements.initiate,
        )


class AsyncActionRequirementsResourceWithRawResponse:
    def __init__(self, action_requirements: AsyncActionRequirementsResource) -> None:
        self._action_requirements = action_requirements

        self.list = async_to_raw_response_wrapper(
            action_requirements.list,
        )
        self.initiate = async_to_raw_response_wrapper(
            action_requirements.initiate,
        )


class ActionRequirementsResourceWithStreamingResponse:
    def __init__(self, action_requirements: ActionRequirementsResource) -> None:
        self._action_requirements = action_requirements

        self.list = to_streamed_response_wrapper(
            action_requirements.list,
        )
        self.initiate = to_streamed_response_wrapper(
            action_requirements.initiate,
        )


class AsyncActionRequirementsResourceWithStreamingResponse:
    def __init__(self, action_requirements: AsyncActionRequirementsResource) -> None:
        self._action_requirements = action_requirements

        self.list = async_to_streamed_response_wrapper(
            action_requirements.list,
        )
        self.initiate = async_to_streamed_response_wrapper(
            action_requirements.initiate,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List

from ..._models import BaseModel
from ..pagination_meta import PaginationMeta
from .auto_resp_config import AutoRespConfig

__all__ = ["AutorespConfigListResponse"]


class AutorespConfigListResponse(BaseModel):
    data: List[AutoRespConfig]

    meta: PaginationMeta



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from .._models import BaseModel
from .pagination_meta import PaginationMeta

__all__ = ["MessagingURLDomainListResponse", "Data"]


class Data(BaseModel):
    id: Optional[str] = None

    record_type: Optional[str] = None

    url_domain: Optional[str] = None

    use_case: Optional[str] = None


class MessagingURLDomainListResponse(BaseModel):
    data: Optional[List[Data]] = None

    meta: Optional[PaginationMeta] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List

from pydantic import Field as FieldInfo

from .._models import BaseModel

__all__ = ["PhoneNumberAssignmentByProfileRetrievePhoneNumberStatusResponse", "Record"]


class Record(BaseModel):
    phone_number: str = FieldInfo(alias="phoneNumber")
    """The phone number that the status is being checked for."""

    status: str
    """The status of the associated phone number assignment."""

    task_id: str = FieldInfo(alias="taskId")
    """The ID of the task associated with the phone number."""


class PhoneNumberAssignmentByProfileRetrievePhoneNumberStatusResponse(BaseModel):
    records: List[Record]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["CallEventListParams", "Filter", "FilterApplicationName", "FilterOccurredAt", "Page"]


class CallEventListParams(TypedDict, total=False):
    filter: Filter
    """Consolidated filter parameter (deepObject style).

    Originally: filter[application_name][contains],
    filter[outbound.outbound_voice_profile_id], filter[leg_id],
    filter[application_session_id], filter[connection_id], filter[product],
    filter[failed], filter[from], filter[to], filter[name], filter[type],
    filter[occurred_at][eq/gt/gte/lt/lte], filter[status]
    """

    page: Page
    """Consolidated page parameter (deepObject style).

    Originally: page[after], page[before], page[limit], page[size], page[number]
    """


class FilterApplicationName(TypedDict, total=False):
    contains: str
    """
    If present, applications with <code>application_name</code> containing the given
    value will be returned. Matching is not case-sensitive. Requires at least three
    characters.
    """


class FilterOccurredAt(TypedDict, total=False):
    eq: str
    """Event occurred_at: equal"""

    gt: str
    """Event occurred_at: greater than"""

    gte: str
    """Event occurred_at: greater than or equal"""

    lt: str
    """Event occurred_at: lower than"""

    lte: str
    """Event occurred_at: lower than or equal"""


_FilterReservedKeywords = TypedDict(
    "_FilterReservedKeywords",
    {
        "from": str,
    },
    total=False,
)


class Filter(_FilterReservedKeywords, total=False):
    application_name: FilterApplicationName
    """Application name filters"""

    application_session_id: str
    """The unique identifier of the call session.

    A session may include multiple call leg events.
    """

    connection_id: str
    """The unique identifier of the conection."""

    failed: bool
    """Delivery failed or not."""

    leg_id: str
    """The unique identifier of an individual call leg."""

    name: str
    """
    If present, conferences will be filtered to those with a matching `name`
    attribute. Matching is case-sensitive
    """

    occurred_at: FilterOccurredAt
    """Event occurred_at filters"""

    outbound_outbound_voice_profile_id: Annotated[str, PropertyInfo(alias="outbound.outbound_voice_profile_id")]
    """Identifies the associated outbound voice profile."""

    product: Literal["call_control", "fax", "texml"]
    """Filter by product."""

    status: Literal["init", "in_progress", "completed"]
    """If present, conferences will be filtered by status."""

    to: str
    """Filter by To number."""

    type: Literal["command", "webhook"]
    """Event type"""


class Page(TypedDict, total=False):
    after: str
    """Opaque identifier of next page"""

    before: str
    """Opaque identifier of previous page"""

    limit: int
    """Limit of records per single page"""

    number: int
    """The page number to load"""

    size: int
    """The size of the page"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Required, TypedDict

from .._types import SequenceNotStr
from .service_plan import ServicePlan
from .traffic_type import TrafficType
from .usage_payment_method import UsagePaymentMethod
from .outbound_call_recording_param import OutboundCallRecordingParam

__all__ = ["OutboundVoiceProfileUpdateParams", "CallingWindow"]


class OutboundVoiceProfileUpdateParams(TypedDict, total=False):
    name: Required[str]
    """A user-supplied name to help with organization."""

    billing_group_id: Optional[str]
    """The ID of the billing group associated with the outbound proflile.

    Defaults to null (for no group assigned).
    """

    call_recording: OutboundCallRecordingParam

    calling_window: CallingWindow
    """
    (BETA) Specifies the time window and call limits for calls made using this
    outbound voice profile.
    """

    concurrent_call_limit: Optional[int]
    """Must be no more than your global concurrent call limit. Null means no limit."""

    daily_spend_limit: str
    """
    The maximum amount of usage charges, in USD, you want Telnyx to allow on this
    outbound voice profile in a day before disallowing new calls.
    """

    daily_spend_limit_enabled: bool
    """
    Specifies whether to enforce the daily_spend_limit on this outbound voice
    profile.
    """

    enabled: bool
    """Specifies whether the outbound voice profile can be used.

    Disabled profiles will result in outbound calls being blocked for the associated
    Connections.
    """

    max_destination_rate: float
    """
    Maximum rate (price per minute) for a Destination to be allowed when making
    outbound calls.
    """

    service_plan: ServicePlan
    """Indicates the coverage of the termination regions."""

    tags: SequenceNotStr[str]

    traffic_type: TrafficType
    """Specifies the type of traffic allowed in this profile."""

    usage_payment_method: UsagePaymentMethod
    """Setting for how costs for outbound profile are calculated."""

    whitelisted_destinations: SequenceNotStr[str]
    """
    The list of destinations you want to be able to call using this outbound voice
    profile formatted in alpha2.
    """


class CallingWindow(TypedDict, total=False):
    calls_per_cld: int
    """
    (BETA) The maximum number of calls that can be initiated to a single called
    party (CLD) within the calling window. A null value means no limit.
    """

    end_time: str
    """
    (BETA) The UTC time of day (in HH:MM format, 24-hour clock) when calls are no
    longer allowed to start.
    """

    start_time: str
    """
    (BETA) The UTC time of day (in HH:MM format, 24-hour clock) when calls are
    allowed to start.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime
from typing_extensions import Literal

from .._models import BaseModel

__all__ = ["OAuthGrant"]


class OAuthGrant(BaseModel):
    id: str
    """Unique identifier for the OAuth grant"""

    client_id: str
    """OAuth client identifier"""

    created_at: datetime
    """Timestamp when the grant was created"""

    record_type: Literal["oauth_grant"]
    """Record type identifier"""

    scopes: List[str]
    """List of granted OAuth scopes"""

    last_used_at: Optional[datetime] = None
    """Timestamp when the grant was last used"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .call_list_params import CallListParams as CallListParams
from .call_list_response import CallListResponse as CallListResponse
from .call_retrieve_response import CallRetrieveResponse as CallRetrieveResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from telnyx import Telnyx, AsyncTelnyx
from tests.utils import assert_matches_type
from telnyx.types import (
    OutboundVoiceProfileListResponse,
    OutboundVoiceProfileCreateResponse,
    OutboundVoiceProfileDeleteResponse,
    OutboundVoiceProfileUpdateResponse,
    OutboundVoiceProfileRetrieveResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestOutboundVoiceProfiles:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_create(self, client: Telnyx) -> None:
        outbound_voice_profile = client.outbound_voice_profiles.create(
            name="office",
        )
        assert_matches_type(OutboundVoiceProfileCreateResponse, outbound_voice_profile, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_create_with_all_params(self, client: Telnyx) -> None:
        outbound_voice_profile = client.outbound_voice_profiles.create(
            name="office",
            billing_group_id="6a09cdc3-8948-47f0-aa62-74ac943d6c58",
            call_recording={
                "call_recording_caller_phone_numbers": ["+19705555098"],
                "call_recording_channels": "dual",
                "call_recording_format": "mp3",
                "call_recording_type": "by_caller_phone_number",
            },
            calling_window={
                "calls_per_cld": 5,
                "end_time": "18:11:19.117Z",
                "start_time": "18:11:19.117Z",
            },
            concurrent_call_limit=10,
            daily_spend_limit="100.00",
            daily_spend_limit_enabled=True,
            enabled=True,
            max_destination_rate=10,
            service_plan="global",
            tags=["office-profile"],
            traffic_type="conversational",
            usage_payment_method="rate-deck",
            whitelisted_destinations=["US", "BR", "AU"],
        )
        assert_matches_type(OutboundVoiceProfileCreateResponse, outbound_voice_profile, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_create(self, client: Telnyx) -> None:
        response = client.outbound_voice_profiles.with_raw_response.create(
            name="office",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        outbound_voice_profile = response.parse()
        assert_matches_type(OutboundVoiceProfileCreateResponse, outbound_voice_profile, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_create(self, client: Telnyx) -> None:
        with client.outbound_voice_profiles.with_streaming_response.create(
            name="office",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            outbound_voice_profile = response.parse()
            assert_matches_type(OutboundVoiceProfileCreateResponse, outbound_voice_profile, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_retrieve(self, client: Telnyx) -> None:
        outbound_voice_profile = client.outbound_voice_profiles.retrieve(
            "1293384261075731499",
        )
        assert_matches_type(OutboundVoiceProfileRetrieveResponse, outbound_voice_profile, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_retrieve(self, client: Telnyx) -> None:
        response = client.outbound_voice_profiles.with_raw_response.retrieve(
            "1293384261075731499",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        outbound_voice_profile = response.parse()
        assert_matches_type(OutboundVoiceProfileRetrieveResponse, outbound_voice_profile, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_retrieve(self, client: Telnyx) -> None:
        with client.outbound_voice_profiles.with_streaming_response.retrieve(
            "1293384261075731499",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            outbound_voice_profile = response.parse()
            assert_matches_type(OutboundVoiceProfileRetrieveResponse, outbound_voice_profile, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_retrieve(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.outbound_voice_profiles.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_update(self, client: Telnyx) -> None:
        outbound_voice_profile = client.outbound_voice_profiles.update(
            id="1293384261075731499",
            name="office",
        )
        assert_matches_type(OutboundVoiceProfileUpdateResponse, outbound_voice_profile, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_update_with_all_params(self, client: Telnyx) -> None:
        outbound_voice_profile = client.outbound_voice_profiles.update(
            id="1293384261075731499",
            name="office",
            billing_group_id="6a09cdc3-8948-47f0-aa62-74ac943d6c58",
            call_recording={
                "call_recording_caller_phone_numbers": ["+19705555098"],
                "call_recording_channels": "dual",
                "call_recording_format": "mp3",
                "call_recording_type": "by_caller_phone_number",
            },
            calling_window={
                "calls_per_cld": 5,
                "end_time": "18:11:19.117Z",
                "start_time": "18:11:19.117Z",
            },
            concurrent_call_limit=10,
            daily_spend_limit="100.00",
            daily_spend_limit_enabled=True,
            enabled=True,
            max_destination_rate=10,
            service_plan="global",
            tags=["office-profile"],
            traffic_type="conversational",
            usage_payment_method="rate-deck",
            whitelisted_destinations=["US", "BR", "AU"],
        )
        assert_matches_type(OutboundVoiceProfileUpdateResponse, outbound_voice_profile, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_update(self, client: Telnyx) -> None:
        response = client.outbound_voice_profiles.with_raw_response.update(
            id="1293384261075731499",
            name="office",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        outbound_voice_profile = response.parse()
        assert_matches_type(OutboundVoiceProfileUpdateResponse, outbound_voice_profile, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_update(self, client: Telnyx) -> None:
        with client.outbound_voice_profiles.with_streaming_response.update(
            id="1293384261075731499",
            name="office",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            outbound_voice_profile = response.parse()
            assert_matches_type(OutboundVoiceProfileUpdateResponse, outbound_voice_profile, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_update(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.outbound_voice_profiles.with_raw_response.update(
                id="",
                name="office",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list(self, client: Telnyx) -> None:
        outbound_voice_profile = client.outbound_voice_profiles.list()
        assert_matches_type(OutboundVoiceProfileListResponse, outbound_voice_profile, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list_with_all_params(self, client: Telnyx) -> None:
        outbound_voice_profile = client.outbound_voice_profiles.list(
            filter={"name": {"contains": "office-profile"}},
            page={
                "number": 1,
                "size": 1,
            },
            sort="name",
        )
        assert_matches_type(OutboundVoiceProfileListResponse, outbound_voice_profile, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_list(self, client: Telnyx) -> None:
        response = client.outbound_voice_profiles.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        outbound_voice_profile = response.parse()
        assert_matches_type(OutboundVoiceProfileListResponse, outbound_voice_profile, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_list(self, client: Telnyx) -> None:
        with client.outbound_voice_profiles.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            outbound_voice_profile = response.parse()
            assert_matches_type(OutboundVoiceProfileListResponse, outbound_voice_profile, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_delete(self, client: Telnyx) -> None:
        outbound_voice_profile = client.outbound_voice_profiles.delete(
            "1293384261075731499",
        )
        assert_matches_type(OutboundVoiceProfileDeleteResponse, outbound_voice_profile, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_delete(self, client: Telnyx) -> None:
        response = client.outbound_voice_profiles.with_raw_response.delete(
            "1293384261075731499",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        outbound_voice_profile = response.parse()
        assert_matches_type(OutboundVoiceProfileDeleteResponse, outbound_voice_profile, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_delete(self, client: Telnyx) -> None:
        with client.outbound_voice_profiles.with_streaming_response.delete(
            "1293384261075731499",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            outbound_voice_profile = response.parse()
            assert_matches_type(OutboundVoiceProfileDeleteResponse, outbound_voice_profile, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_delete(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.outbound_voice_profiles.with_raw_response.delete(
                "",
            )


class TestAsyncOutboundVoiceProfiles:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_create(self, async_client: AsyncTelnyx) -> None:
        outbound_voice_profile = await async_client.outbound_voice_profiles.create(
            name="office",
        )
        assert_matches_type(OutboundVoiceProfileCreateResponse, outbound_voice_profile, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncTelnyx) -> None:
        outbound_voice_profile = await async_client.outbound_voice_profiles.create(
            name="office",
            billing_group_id="6a09cdc3-8948-47f0-aa62-74ac943d6c58",
            call_recording={
                "call_recording_caller_phone_numbers": ["+19705555098"],
                "call_recording_channels": "dual",
                "call_recording_format": "mp3",
                "call_recording_type": "by_caller_phone_number",
            },
            calling_window={
                "calls_per_cld": 5,
                "end_time": "18:11:19.117Z",
                "start_time": "18:11:19.117Z",
            },
            concurrent_call_limit=10,
            daily_spend_limit="100.00",
            daily_spend_limit_enabled=True,
            enabled=True,
            max_destination_rate=10,
            service_plan="global",
            tags=["office-profile"],
            traffic_type="conversational",
            usage_payment_method="rate-deck",
            whitelisted_destinations=["US", "BR", "AU"],
        )
        assert_matches_type(OutboundVoiceProfileCreateResponse, outbound_voice_profile, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_create(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.outbound_voice_profiles.with_raw_response.create(
            name="office",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        outbound_voice_profile = await response.parse()
        assert_matches_type(OutboundVoiceProfileCreateResponse, outbound_voice_profile, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncTelnyx) -> None:
        async with async_client.outbound_voice_profiles.with_streaming_response.create(
            name="office",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            outbound_voice_profile = await response.parse()
            assert_matches_type(OutboundVoiceProfileCreateResponse, outbound_voice_profile, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_retrieve(self, async_client: AsyncTelnyx) -> None:
        outbound_voice_profile = await async_client.outbound_voice_profiles.retrieve(
            "1293384261075731499",
        )
        assert_matches_type(OutboundVoiceProfileRetrieveResponse, outbound_voice_profile, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.outbound_voice_profiles.with_raw_response.retrieve(
            "1293384261075731499",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        outbound_voice_profile = await response.parse()
        assert_matches_type(OutboundVoiceProfileRetrieveResponse, outbound_voice_profile, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncTelnyx) -> None:
        async with async_client.outbound_voice_profiles.with_streaming_response.retrieve(
            "1293384261075731499",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            outbound_voice_profile = await response.parse()
            assert_matches_type(OutboundVoiceProfileRetrieveResponse, outbound_voice_profile, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.outbound_voice_profiles.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_update(self, async_client: AsyncTelnyx) -> None:
        outbound_voice_profile = await async_client.outbound_voice_profiles.update(
            id="1293384261075731499",
            name="office",
        )
        assert_matches_type(OutboundVoiceProfileUpdateResponse, outbound_voice_profile, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_update_with_all_params(self, async_client: AsyncTelnyx) -> None:
        outbound_voice_profile = await async_client.outbound_voice_profiles.update(
            id="1293384261075731499",
            name="office",
            billing_group_id="6a09cdc3-8948-47f0-aa62-74ac943d6c58",
            call_recording={
                "call_recording_caller_phone_numbers": ["+19705555098"],
                "call_recording_channels": "dual",
                "call_recording_format": "mp3",
                "call_recording_type": "by_caller_phone_number",
            },
            calling_window={
                "calls_per_cld": 5,
                "end_time": "18:11:19.117Z",
                "start_time": "18:11:19.117Z",
            },
            concurrent_call_limit=10,
            daily_spend_limit="100.00",
            daily_spend_limit_enabled=True,
            enabled=True,
            max_destination_rate=10,
            service_plan="global",
            tags=["office-profile"],
            traffic_type="conversational",
            usage_payment_method="rate-deck",
            whitelisted_destinations=["US", "BR", "AU"],
        )
        assert_matches_type(OutboundVoiceProfileUpdateResponse, outbound_voice_profile, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_update(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.outbound_voice_profiles.with_raw_response.update(
            id="1293384261075731499",
            name="office",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        outbound_voice_profile = await response.parse()
        assert_matches_type(OutboundVoiceProfileUpdateResponse, outbound_voice_profile, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_update(self, async_client: AsyncTelnyx) -> None:
        async with async_client.outbound_voice_profiles.with_streaming_response.update(
            id="1293384261075731499",
            name="office",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            outbound_voice_profile = await response.parse()
            assert_matches_type(OutboundVoiceProfileUpdateResponse, outbound_voice_profile, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_update(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.outbound_voice_profiles.with_raw_response.update(
                id="",
                name="office",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list(self, async_client: AsyncTelnyx) -> None:
        outbound_voice_profile = await async_client.outbound_voice_profiles.list()
        assert_matches_type(OutboundVoiceProfileListResponse, outbound_voice_profile, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncTelnyx) -> None:
        outbound_voice_profile = await async_client.outbound_voice_profiles.list(
            filter={"name": {"contains": "office-profile"}},
            page={
                "number": 1,
                "size": 1,
            },
            sort="name",
        )
        assert_matches_type(OutboundVoiceProfileListResponse, outbound_voice_profile, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.outbound_voice_profiles.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        outbound_voice_profile = await response.parse()
        assert_matches_type(OutboundVoiceProfileListResponse, outbound_voice_profile, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncTelnyx) -> None:
        async with async_client.outbound_voice_profiles.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            outbound_voice_profile = await response.parse()
            assert_matches_type(OutboundVoiceProfileListResponse, outbound_voice_profile, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_delete(self, async_client: AsyncTelnyx) -> None:
        outbound_voice_profile = await async_client.outbound_voice_profiles.delete(
            "1293384261075731499",
        )
        assert_matches_type(OutboundVoiceProfileDeleteResponse, outbound_voice_profile, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.outbound_voice_profiles.with_raw_response.delete(
            "1293384261075731499",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        outbound_voice_profile = await response.parse()
        assert_matches_type(OutboundVoiceProfileDeleteResponse, outbound_voice_profile, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncTelnyx) -> None:
        async with async_client.outbound_voice_profiles.with_streaming_response.delete(
            "1293384261075731499",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            outbound_voice_profile = await response.parse()
            assert_matches_type(OutboundVoiceProfileDeleteResponse, outbound_voice_profile, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_delete(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.outbound_voice_profiles.with_raw_response.delete(
                "",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options
from ...types.phone_numbers import messaging_list_params, messaging_update_params
from ...types.phone_numbers.messaging_list_response import MessagingListResponse
from ...types.phone_numbers.messaging_update_response import MessagingUpdateResponse
from ...types.phone_numbers.messaging_retrieve_response import MessagingRetrieveResponse

__all__ = ["MessagingResource", "AsyncMessagingResource"]


class MessagingResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> MessagingResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return MessagingResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> MessagingResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return MessagingResourceWithStreamingResponse(self)

    def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> MessagingRetrieveResponse:
        """
        Retrieve a phone number with messaging settings

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/phone_numbers/{id}/messaging",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=MessagingRetrieveResponse,
        )

    def update(
        self,
        id: str,
        *,
        messaging_product: str | Omit = omit,
        messaging_profile_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> MessagingUpdateResponse:
        """
        Update the messaging profile and/or messaging product of a phone number

        Args:
          messaging_product:
              Configure the messaging product for this number:

              - Omit this field or set its value to `null` to keep the current value.
              - Set this field to a quoted product ID to set this phone number to that product

          messaging_profile_id:
              Configure the messaging profile this phone number is assigned to:

              - Omit this field or set its value to `null` to keep the current value.
              - Set this field to `""` to unassign the number from its messaging profile
              - Set this field to a quoted UUID of a messaging profile to assign this number
                to that messaging profile

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._patch(
            f"/phone_numbers/{id}/messaging",
            body=maybe_transform(
                {
                    "messaging_product": messaging_product,
                    "messaging_profile_id": messaging_profile_id,
                },
                messaging_update_params.MessagingUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=MessagingUpdateResponse,
        )

    def list(
        self,
        *,
        page: messaging_list_params.Page | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> MessagingListResponse:
        """
        List phone numbers with messaging settings

        Args:
          page: Consolidated page parameter (deepObject style). Originally: page[number],
              page[size]

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/phone_numbers/messaging",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"page": page}, messaging_list_params.MessagingListParams),
            ),
            cast_to=MessagingListResponse,
        )


class AsyncMessagingResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncMessagingResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncMessagingResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncMessagingResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncMessagingResourceWithStreamingResponse(self)

    async def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> MessagingRetrieveResponse:
        """
        Retrieve a phone number with messaging settings

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/phone_numbers/{id}/messaging",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=MessagingRetrieveResponse,
        )

    async def update(
        self,
        id: str,
        *,
        messaging_product: str | Omit = omit,
        messaging_profile_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> MessagingUpdateResponse:
        """
        Update the messaging profile and/or messaging product of a phone number

        Args:
          messaging_product:
              Configure the messaging product for this number:

              - Omit this field or set its value to `null` to keep the current value.
              - Set this field to a quoted product ID to set this phone number to that product

          messaging_profile_id:
              Configure the messaging profile this phone number is assigned to:

              - Omit this field or set its value to `null` to keep the current value.
              - Set this field to `""` to unassign the number from its messaging profile
              - Set this field to a quoted UUID of a messaging profile to assign this number
                to that messaging profile

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._patch(
            f"/phone_numbers/{id}/messaging",
            body=await async_maybe_transform(
                {
                    "messaging_product": messaging_product,
                    "messaging_profile_id": messaging_profile_id,
                },
                messaging_update_params.MessagingUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=MessagingUpdateResponse,
        )

    async def list(
        self,
        *,
        page: messaging_list_params.Page | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> MessagingListResponse:
        """
        List phone numbers with messaging settings

        Args:
          page: Consolidated page parameter (deepObject style). Originally: page[number],
              page[size]

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/phone_numbers/messaging",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform({"page": page}, messaging_list_params.MessagingListParams),
            ),
            cast_to=MessagingListResponse,
        )


class MessagingResourceWithRawResponse:
    def __init__(self, messaging: MessagingResource) -> None:
        self._messaging = messaging

        self.retrieve = to_raw_response_wrapper(
            messaging.retrieve,
        )
        self.update = to_raw_response_wrapper(
            messaging.update,
        )
        self.list = to_raw_response_wrapper(
            messaging.list,
        )


class AsyncMessagingResourceWithRawResponse:
    def __init__(self, messaging: AsyncMessagingResource) -> None:
        self._messaging = messaging

        self.retrieve = async_to_raw_response_wrapper(
            messaging.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            messaging.update,
        )
        self.list = async_to_raw_response_wrapper(
            messaging.list,
        )


class MessagingResourceWithStreamingResponse:
    def __init__(self, messaging: MessagingResource) -> None:
        self._messaging = messaging

        self.retrieve = to_streamed_response_wrapper(
            messaging.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            messaging.update,
        )
        self.list = to_streamed_response_wrapper(
            messaging.list,
        )


class AsyncMessagingResourceWithStreamingResponse:
    def __init__(self, messaging: AsyncMessagingResource) -> None:
        self._messaging = messaging

        self.retrieve = async_to_streamed_response_wrapper(
            messaging.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            messaging.update,
        )
        self.list = async_to_streamed_response_wrapper(
            messaging.list,
        )
